import * as React from 'react';
import { Widget } from '@lumino/widgets';
import { showDialog, Dialog, showErrorMessage, ReactWidget, } from '@jupyterlab/apputils';
import { refreshIcon } from '@jupyterlab/ui-components';
import { PathExt } from '@jupyterlab/coreutils';
import { ERROR_TYPES_MAP } from '../../linter/reports';
import { ErrorTypeKeys } from '../../linter/interfaces';
import { configIcon, requirermentsIcon, listIcon, cellIcon, typeIcon, iconMap, eyeIcon, filterIcon, } from '../../iconimports';
import { createJulynterConfigWidget } from './julynterconfigwidget';
/**
 * A widget used to rename a file.
 */
class RequirementsHandler extends Widget {
    /**
     * Construct a new "rename" dialog.
     */
    constructor(oldPath) {
        const body = document.createElement('div');
        const existingLabel = document.createElement('label');
        existingLabel.textContent = 'Requirements file';
        const existingPath = document.createElement('span');
        existingPath.textContent = oldPath;
        const nameTitle = document.createElement('label');
        nameTitle.textContent = 'New Name';
        nameTitle.className = 'jp-new-name-title';
        const name = document.createElement('input');
        body.appendChild(existingLabel);
        body.appendChild(existingPath);
        body.appendChild(nameTitle);
        body.appendChild(name);
        super({ node: body });
        this.addClass('jp-FileDialog');
        const ext = PathExt.extname(oldPath);
        const value = (this.inputNode.value = PathExt.basename(oldPath));
        this.inputNode.setSelectionRange(0, value.length - ext.length);
    }
    /**
     * Get the input text node.
     */
    get inputNode() {
        return this.node.getElementsByTagName('input')[0];
    }
    /**
     * Get the value of the widget.
     */
    getValue() {
        return this.inputNode.value;
    }
}
export class ToolbarWidget extends ReactWidget {
    constructor(options) {
        super();
        this.notebook = options.notebook;
        this.labShell = options.labShell;
        this.tracker = options.tracker;
        this.config = options.config;
        this.handlers = options.handlers;
        this.errorHandler = options.errorHandler;
        this.filtered = options.filtered;
        this.addClass('julynter-toolbar-widget');
    }
    toggle(key) {
        try {
            const options = this.notebook.options;
            options.updateType(key, !options.checkType(key));
        }
        catch (error) {
            throw this.errorHandler.report(error, 'ToolbarWidget:toggle', [key]);
        }
    }
    toggleMode() {
        try {
            let mode = this.notebook.options.checkMode();
            if (mode === 'list') {
                mode = 'cell';
            }
            else if (mode === 'cell') {
                mode = 'type';
            }
            else {
                mode = 'list';
            }
            this.notebook.options.updateMode(mode);
        }
        catch (error) {
            throw this.errorHandler.report(error, 'ToolbarWidget:toggleMode', []);
        }
    }
    toggleView() {
        try {
            const options = this.notebook.options;
            options.updateView(!options.checkView());
        }
        catch (error) {
            throw this.errorHandler.report(error, 'ToolbarWidget:toggleView', []);
        }
    }
    toggleRestart() {
        try {
            const options = this.notebook.options;
            options.updateRestart(!options.checkRestart());
        }
        catch (error) {
            throw this.errorHandler.report(error, 'ToolbarWidget:toggleRestart', []);
        }
    }
    removeFilters() {
        try {
            const options = this.notebook.options;
            options.resetFiltered();
        }
        catch (error) {
            throw this.errorHandler.report(error, 'ToolbarWidget:toggleRestart', []);
        }
    }
    changeRequirements() {
        try {
            const options = this.notebook.options;
            return showDialog({
                title: 'Set requirements file',
                body: new RequirementsHandler(options.checkRequirements()),
                focusNodeSelector: 'input',
                buttons: [Dialog.cancelButton(), Dialog.okButton({ label: 'Set' })],
            }).then((result) => {
                if (!result.value) {
                    return;
                }
                const name = result.value;
                if (name.length === 0 || name.includes(':')) {
                    void showErrorMessage('Set Requirements Error', Error(`"${result.value}" is not a valid name for a file. ` +
                        'Names must have nonzero length, ' +
                        'and cannot include ":"'));
                    return null;
                }
                options.updateRequirements(name);
            });
        }
        catch (error) {
            throw this.errorHandler.report(error, 'ToolbarWidget:changeRequirements', []);
        }
    }
    configure() {
        try {
            this.labShell.add(createJulynterConfigWidget(this.tracker, this.handlers, this.config, this.notebook, this.errorHandler), 'main');
        }
        catch (error) {
            throw this.errorHandler.report(error, 'ToolbarWidget:configure', []);
        }
    }
    createFilterButtons() {
        let i = 0;
        return ErrorTypeKeys.map((key) => {
            const element = ERROR_TYPES_MAP[key];
            const toggleClass = this.notebook.options.checkType(key)
                ? 'julynter-toolbar-icon-selected'
                : 'julynter-toolbar-icon';
            const icon = iconMap[key];
            return (React.createElement("div", { key: `toolbar-${key}-${i++}`, className: "julynter-toolbar-button", title: element.label, onClick: () => this.toggle(key) },
                React.createElement(icon.react, { elementSize: "normal", className: toggleClass, elementPosition: "center" })));
        });
    }
    createViewButton() {
        const toggleClass = this.notebook.options.checkView()
            ? 'julynter-toolbar-icon-selected'
            : 'julynter-toolbar-icon';
        return (React.createElement("div", { className: "julynter-toolbar-button", title: "Toggle view on cells", onClick: this.toggleView.bind(this) },
            React.createElement(eyeIcon.react, { elementSize: "normal", className: toggleClass, elementPosition: "center" })));
    }
    createRestartButton() {
        const toggleClass = this.notebook.options.checkRestart()
            ? 'julynter-toolbar-icon-selected'
            : 'julynter-toolbar-icon';
        return (React.createElement("div", { className: "julynter-toolbar-button", title: "Toggle lints that require a kernel restart", onClick: this.toggleRestart.bind(this) },
            React.createElement(refreshIcon.react, { elementSize: "normal", className: toggleClass, elementPosition: "center" })));
    }
    createRestartFilterButton() {
        let toggleClass = 'julynter-toolbar-icon';
        let element = (React.createElement("span", null,
            " ",
            this.filtered.length > 99 ? '++' : this.filtered.length,
            " "));
        if (this.filtered.length === 0) {
            toggleClass = 'julynter-toolbar-icon-selected';
            element = React.createElement("span", null);
        }
        return (React.createElement("div", { className: "julynter-toolbar-button", title: "Remove all individual lint filters", onClick: this.removeFilters.bind(this) },
            React.createElement("div", { className: "julynter-toolbar-icon-with-text" },
                React.createElement(filterIcon.react, { elementSize: "normal", className: toggleClass, elementPosition: "center" }),
                React.createElement("span", { className: "julynter-toolbar-icon-text" }, element))));
    }
    createModeButton() {
        let toggleIcon = null;
        const mode = this.notebook.options.checkMode();
        if (mode === 'list') {
            toggleIcon = (React.createElement(listIcon.react, { elementSize: "normal", className: "julynter-toolbar-icon", elementPosition: "center" }));
        }
        else if (mode === 'cell') {
            toggleIcon = (React.createElement(cellIcon.react, { elementSize: "normal", className: "julynter-toolbar-icon", elementPosition: "center" }));
        }
        else if (mode === 'type') {
            toggleIcon = (React.createElement(typeIcon.react, { elementSize: "normal", className: "julynter-toolbar-icon", elementPosition: "center" }));
        }
        return (React.createElement("div", { key: "toolbar-mode", className: "julynter-toolbar-button", title: "Alternate Mode", onClick: this.toggleMode.bind(this) }, toggleIcon));
    }
    render() {
        try {
            return (React.createElement("div", null,
                React.createElement("div", { className: 'julynter-toolbar' },
                    this.createFilterButtons(),
                    this.createViewButton(),
                    this.createRestartButton(),
                    this.createRestartFilterButton(),
                    this.createModeButton(),
                    React.createElement("div", { key: "toolbar-req", title: "Change requirements location", className: "julynter-toolbar-button", onClick: this.changeRequirements.bind(this) },
                        React.createElement(requirermentsIcon.react, { elementSize: "normal", className: "julynter-toolbar-icon", elementPosition: "center" })),
                    React.createElement("div", { key: "toolbar-config", title: "Configure Julynter", className: "julynter-toolbar-button", onClick: this.configure.bind(this) },
                        React.createElement(configIcon.react, { elementSize: "normal", className: "julynter-toolbar-icon", elementPosition: "center" })))));
        }
        catch (error) {
            throw this.errorHandler.report(error, 'ToolbarWidget:render', []);
        }
    }
}
